/*======================================================*/
/*														*/
/*	straightFrameEx.cpp									*/
/*						(c) HAL 2009-					*/
/*														*/
/*======================================================*/
#include "straightFrameEx.h"

VCNT_RESULT	straightFrameEx::getAperiodicity( vector<double>& buffer )
{
	VCNT_RESULT	result = VCNT_INVALID_CALL;
	if( !( aperiodicityBuffer.empty() ) ){
		buffer.resize( aperiodicityBuffer.size() );
		for( unsigned long i = 0; i < aperiodicityBuffer.size(); i++ )
			buffer[i] = aperiodicityBuffer[i];
		result = VCNT_TRUE;
	}
	return result;
}

VCNT_RESULT	straightFrameEx::getAperiodicity( double* buffer, unsigned long bufferLength )
{
	VCNT_RESULT	result = VCNT_INVALID_CALL;
	if( !( aperiodicityBuffer.empty() ) && bufferLength >= aperiodicityBuffer.size()){
		for( unsigned long i = 0; i < aperiodicityBuffer.size(); i++ )
			buffer[i] = aperiodicityBuffer[i];
		result = VCNT_TRUE;
	}
	return result;
}

VCNT_RESULT	straightFrameEx::getSpectrum( vector<double>& buffer )
{
	VCNT_RESULT	result = VCNT_INVALID_CALL;
	if( !( spectrumBuffer.empty() ) ){
		buffer.resize( spectrumBuffer.size() );
		for( unsigned long i = 0; i < spectrumBuffer.size(); i++ )
			buffer[i] = spectrumBuffer[i];
		result = VCNT_TRUE;
	}
	return result;
}

VCNT_RESULT	straightFrameEx::getSpectrum( double* buffer, unsigned long bufferLength )
{
	VCNT_RESULT	result = VCNT_INVALID_CALL;
	if( !( spectrumBuffer.empty() ) && bufferLength >= spectrumBuffer.size()){
		for( unsigned long i = 0; i < spectrumBuffer.size(); i++ )
			buffer[i] = spectrumBuffer[i];
		result = VCNT_TRUE;
	}
	return result;
}

VCNT_RESULT	straightFrameEx::setAperiodicity( vector<double>& buffer )
{
	VCNT_RESULT	result = VCNT_INVALID_CALL;
	if( !( buffer.empty() ) ){
		aperiodicityBuffer.resize( buffer.size() );
		for( unsigned long i = 0; i < buffer.size(); i++ )
			aperiodicityBuffer[i] = buffer[i];
		result = VCNT_TRUE;
	}
	return result;
}

VCNT_RESULT	straightFrameEx::setAperiodicity( double* buffer, unsigned long bufferLength )
{
	VCNT_RESULT	result = VCNT_INVALID_CALL;
	if( buffer && bufferLength > 0 ){
		aperiodicityBuffer.resize( bufferLength );
		for( unsigned long i = 0; i < bufferLength; i++ )
			aperiodicityBuffer[i] = buffer[i];
		result = VCNT_TRUE;
	}
	return result;
}

VCNT_RESULT	straightFrameEx::setSpectrum( vector<double>& buffer )
{
	VCNT_RESULT	result = VCNT_INVALID_CALL;
	if( !( buffer.empty() ) ){
		spectrumBuffer.resize( buffer.size() );
		for( unsigned long i = 0; i < buffer.size(); i++ )
			spectrumBuffer[i] = buffer[i];
		result = VCNT_TRUE;
	}
	return result;
}

VCNT_RESULT	straightFrameEx::setSpectrum( double* buffer, unsigned long bufferLength )
{
	VCNT_RESULT	result = VCNT_INVALID_CALL;
	if( buffer && bufferLength > 0 ){
		spectrumBuffer.resize( bufferLength );
		for( unsigned long i = 0; i < bufferLength; i++ )
			spectrumBuffer[i] = buffer[i];
		result = VCNT_TRUE;
	}
	return result;
}

VCNT_RESULT	straightFrameEx::setBufferLength( unsigned long aperiodicityLength, unsigned long spectrumLength )
{
	VCNT_RESULT result = VCNT_INVALID_CALL;

	if( aperiodicityLength != 0 && spectrumLength != 0 ){
		vector<double> apTemp;
		vector<double> spTemp;
		apTemp.resize( aperiodicityLength, 0.0 );
		spTemp.resize( spectrumLength, 0.0 );

		double dIndex;
		double rate;
		long nIndex;

		if( !( aperiodicityBuffer.empty() ) ){
			for( unsigned long i = 0; i < apTemp.size(); i++ ){
				dIndex = (double)i * ( (double)(aperiodicityBuffer.size()) / (double)aperiodicityLength );
				nIndex = (long)dIndex;
				rate = dIndex - (double)nIndex;

				if( nIndex > aperiodicityBuffer.size() - 2 )
					apTemp[i] = aperiodicityBuffer[nIndex];
				else
					apTemp[i] = aperiodicityBuffer[nIndex] * ( 1 - rate ) + aperiodicityBuffer[nIndex+1] * rate;
			}
		}

		if( !( spectrumBuffer.empty() ) ){
			for( unsigned long i = 0; i < spTemp.size(); i++ ){
				dIndex = (double)i * ( (double)(spectrumBuffer.size()) / (double)spectrumLength );
				nIndex = (long)dIndex;
				rate = dIndex - (double)nIndex;

				if( nIndex > spectrumBuffer.size() - 2 )
					spTemp[i] = spectrumBuffer[nIndex];
				else
					spTemp[i] = pow( spectrumBuffer[nIndex], ( 1 - rate ) ) * pow( spectrumBuffer[nIndex+1], rate );
			}
		}
		aperiodicityBuffer.swap( apTemp );
		spectrumBuffer.swap( spTemp );
		result = VCNT_TRUE;
	} else {
		aperiodicityBuffer.clear();
		spectrumBuffer.clear();
	}

	return result;
}

VCNT_RESULT	straightFrameEx::applyExpression( vConnectExpression &expression )
{
	VCNT_RESULT	result = VCNT_INVALID_CALL;
	
	if( !( aperiodicityBuffer.empty() ) && !( spectrumBuffer.empty() ) ){
		bool temp = true;
		temp &= ( applyAperiodicityAmplitude( expression.aperiodicityAmplitudeRate ) == VCNT_TRUE );
//		temp &= ( applyDipFilters( expression.dipFilters ) == VCNT_TRUE );
		temp &= ( applySpectrumExtention( expression.spectrumExtentionRate ) == VCNT_TRUE );
		if( temp )
			result = VCNT_TRUE;
		else
			result = VCNT_FALSE;
	}

	return result;
}

VCNT_RESULT	straightFrameEx::applySpectrumExtention( double rate )
{
	VCNT_RESULT result = VCNT_INVALID_CALL;

	if( rate >= 0.0 ){
		unsigned long apLength = (unsigned long)( rate * (double)(aperiodicityBuffer.size()) );
		unsigned long spLength = (unsigned long)( rate * (double)(spectrumBuffer.size()) );
		unsigned long apSrcLength = aperiodicityBuffer.size();
		unsigned long spSrcLength = spectrumBuffer.size();

		result = setBufferLength( apLength, spLength );
		if( result == VCNT_TRUE ){
			aperiodicityBuffer.resize( apSrcLength, 0.0 );
			spectrumBuffer.resize( spSrcLength, 0.0 );
		}
	}
	return result;
}

VCNT_RESULT	straightFrameEx::applyAperiodicityAmplitude( double rate )
{
	VCNT_RESULT result = VCNT_INVALID_CALL;

	if( rate > 0.0 ){
		for( unsigned long i = 0; i < aperiodicityBuffer.size(); i++ )
			aperiodicityBuffer[i] *= rate;
		for( unsigned long i = 0; i < spectrumBuffer.size(); i++ )
			spectrumBuffer[i] *= rate;
		result = VCNT_TRUE;
	}

	return result;
}

VCNT_RESULT	straightFrameEx::applyDipFilters( vector<straightDipFilter>& filters )
{
	VCNT_RESULT result = VCNT_INVALID_CALL;

	if( !( aperiodicityBuffer.empty() ) && !( spectrumBuffer.empty() ) && !( filters.empty() ) ){

		double apFreqShift = ( config.samplingFrequency / 2.0 ) / (double)( aperiodicityBuffer.size() );
		double spFreqShift = ( config.samplingFrequency / 2.0 ) / (double)( spectrumBuffer.size() );

		for( unsigned int i = 0; i < filters.size(); i++ ){
			for( unsigned long j = 0; j < aperiodicityBuffer.size(); j++ )
				aperiodicityBuffer[j] *= filters[i].getFilter( (double)j * apFreqShift );
			for( unsigned long j = 0; j < spectrumBuffer.size(); j++ )
				spectrumBuffer[j] *= filters[i].getFilter( (double)j * spFreqShift );
		}
	}

	return result;
}

VCNT_RESULT straightFrameEx::multipleBuffer( double rate )
{
	for( unsigned long i = 0; i < aperiodicityBuffer.size(); i++ )
		aperiodicityBuffer[i] *= rate;
	for( unsigned long i = 0; i < spectrumBuffer.size(); i++ )
		spectrumBuffer[i] = pow( spectrumBuffer[i], rate );

	return VCNT_TRUE;
}

VCNT_RESULT	straightFrameEx::setFrameBySimpleMorphFrames( vector<straightFrameEx>& frames )
{
	vector<double> apTemp, spTemp;
	double spDefault = 1.0;

	if( frames.empty() )
		spDefault = 0.0;

	for( unsigned int index = 0; index < aperiodicityBuffer.size(); index++ )
		aperiodicityBuffer[index] = 0.0;
	for( unsigned int index = 0; index < spectrumBuffer.size(); index++ )
		spectrumBuffer[index] = spDefault;

	for( unsigned int i = 0; i < frames.size(); i++ ){
		if( !frames[i].empty() ){
			frames[i].setBufferLength( aperiodicityBuffer.size(), spectrumBuffer.size() );
			frames[i].getAperiodicity( apTemp );
			frames[i].getSpectrum( spTemp );
			for( unsigned int index = 0; index < aperiodicityBuffer.size(); index++ )
				aperiodicityBuffer[index] += apTemp[index];
			for( unsigned int index = 0; index < spectrumBuffer.size(); index++ )
				spectrumBuffer[index] *= spTemp[index];
		}
	}

	return VCNT_TRUE;
}

