/*======================================================*/
/*														*/
/*	straightFrameEx.h									*/
/*						(c) HAL 2009-					*/
/*		straightFrameEx class contains					*/
/*			f0, aperiodicity, and spectrum.				*/
/*														*/
/*======================================================*/
#ifndef __straightFrameEx_h__
#define __straightFrameEx_h__

#include "vConnectObjectBase.h"
#include "vConnectExpression.h"

class straightFrameEx : public vConnectObjectBase {
public:

	VCNT_RESULT		setF0( double f0 ){ this->f0 = f0; return VCNT_TRUE; }
	VCNT_RESULT		setAperiodicity( vector<double>& buffer );
	VCNT_RESULT		setAperiodicity( double* buffer, unsigned long bufferLength );
	VCNT_RESULT		setSpectrum( vector<double>& buffer );
	VCNT_RESULT		setSpectrum( double* buffer, unsigned long bufferLength );

	double			getF0( void ){ return f0; }
	VCNT_RESULT		getAperiodicity( vector<double>& buffer );
	VCNT_RESULT		getAperiodicity( double* buffer, unsigned long bufferLength );
	VCNT_RESULT		getSpectrum( vector<double>& buffer );
	VCNT_RESULT		getSpectrum( double* buffer, unsigned long bufferLength );

	unsigned long	getAperiodicityLength( void ){ return aperiodicityBuffer.size(); }
	unsigned long	getSpectrumLength( void ){ return spectrumBuffer.size(); }

	VCNT_RESULT		setBufferLength( unsigned long aperiodicityLength, unsigned long spectrumLength );

	VCNT_RESULT		applyExpression( vConnectExpression& expression );
	VCNT_RESULT		applySpectrumExtention( double rate );
	VCNT_RESULT		applyAperiodicityAmplitude( double rate );
	VCNT_RESULT		applyDipFilters( vector<straightDipFilter>& filters );

	VCNT_RESULT		multipleBuffer( double rate );
	VCNT_RESULT		setFrameBySimpleMorphFrames( vector<straightFrameEx>& frames );

	bool			empty( void ){ return ( aperiodicityBuffer.empty() || spectrumBuffer.empty() ); }
private:

	double			f0;
	vector<double>	aperiodicityBuffer;
	vector<double>	spectrumBuffer;
};

#endif
