/*======================================================*/
/*														*/
/*	vConnect.cpp										*/
/*						(c) HAL 2009-					*/
/*														*/
/*======================================================*/
#include "vConnect.h"

static stBool callbackFunc(Straight straight, stCallbackType callbackType,
                           void *callbackData, void *userData)
{
    int percent;
    
    if (callbackType == STRAIGHT_F0_PERCENTAGE_CALLBACK) {
        percent = (int)callbackData;
        fprintf(stderr, "STRAIGHT F0: %d %%\r", percent);
        if (percent >= 100) {
            fprintf(stderr, "\n");
        }
    } else if (callbackType == STRAIGHT_AP_PERCENTAGE_CALLBACK) {
        percent = (int)callbackData;
        fprintf(stderr, "STRAIGHT AP: %d %%\r", percent);
        if (percent >= 100) {
            fprintf(stderr, "\n");
        }
    } else if (callbackType == STRAIGHT_SPECGRAM_PERCENTAGE_CALLBACK) {
        percent = (int)callbackData;
        fprintf(stderr, "STRAIGHT spectrogram: %d %%\r", percent);
        if (percent >= 100) {
            fprintf(stderr, "\n");
        }
    } else if (callbackType == STRAIGHT_SYNTH_PERCENTAGE_CALLBACK) {
        percent = (int)callbackData;
        fprintf(stderr, "STRAIGHT synthesis: %d %%\r", percent);
        if (percent >= 100) {
            fprintf(stderr, "\n");
        }
    }
	return ST_TRUE;
}


VCNT_RESULT	vConnect::initialize( void )
{
	VCNT_RESULT result;

#ifdef __STRAIGHT_TANDEM__
	straightInitConfigTandem( &config );
#else
	straightInitConfig( &config );
#endif

	config.samplingFrequency = 44100.0;
	straight = straightInitialize( &config );

	if( straight ){
		straightSetCallbackFunc(straight, STRAIGHT_PERCENTAGE_CALLBACK, callbackFunc, NULL);
		result = VCNT_TRUE;
	}else {
#ifdef DEBUG
        cout << "error@vConnect#initialize; cannot initialize STRAIGHT engine." << endl;
#endif
		result = VCNT_FALSE;
	}

	return result;
}

void	vConnect::destroy( void )
{
	if( straight )
		straightDestroy( straight );
}

VCNT_RESULT vConnect::synthesize( string inputFileName, string outputFileName )
{
	VCNT_RESULT result = VCNT_INVALID_CALL;

	if( converter ){
		if( ( result = converter->getData( inputFileName, data ) ) == VCNT_TRUE ){

			unsigned long numFrames = data.getNumFrames();

			vConnectFrame	vcFrame;
			straightFrameEx	stFrame;
			vector<straightFrameEx> frames;
			vector<double>	dynamics;
			long apLength, spLength;

			dynamics.resize( numFrames );
			initializeFileManager();
			straightBuffer.setNumFrames( numFrames );
			apLength = straightBuffer.getAperiodicityNumFrequencyLength();
			spLength = straightBuffer.getSpecgramNumFrequencyLength();

			stFrame.setBufferLength( apLength, spLength );

			for( unsigned long frameIndex = 0; frameIndex < numFrames; frameIndex++ ){
				data.getFrame( frameIndex, vcFrame );

				fileManager.getFrame( frameIndex, vcFrame.pronounces, frames );

				stFrame.setFrameBySimpleMorphFrames( frames );
				stFrame.applyExpression( vcFrame.expression );
				stFrame.setF0( vcFrame.f0 );

				straightBuffer.setFrame( frameIndex, stFrame );
				dynamics[frameIndex] = vcFrame.expression.volumeAmplitudeRate;
			}

			result = straightBuffer.straightSynthesis( waveBuffer );

			if( result == VCNT_TRUE ){
				double secOffset = data.getOffset();
				waveBuffer.applyDynamics( dynamics );
				waveBuffer.normalize();
				waveBuffer.setOffset( secOffset );
				result = waveBuffer.writeWaveFile( outputFileName );
			}
		}
	}

	return result;
}

VCNT_RESULT	vConnect::initializeFileManager( void )
{
	vConnectFrame frame;
	for( unsigned long frameIndex = 0; frameIndex < data.getNumFrames(); frameIndex++ ){
		data.getFrame( frameIndex, frame );
		fileManager.setPronounces( frameIndex, frame.pronounces );
	}

	return VCNT_TRUE;
}

