/*======================================================*/
/*														*/
/*	vsqConverter.cpp									*/
/*						(c) HAL 2009-					*/
/*														*/
/*======================================================*/
#include "vsqConverter.h"

vsqConverter::vsqConverter()
{
	for( int i = 0; i < NOTE_NUM; i++ )
		noteFrequency[i] = A4_PITCH * pow( 2.0, (double)( i - A4_NOTE ) / 12.0 );
	for( int i = 0; i < VIB_NUM; i++ ){
		double period = exp( 5.24 - 1.07e-2 * i ) * 2.0 / 1000.0;
		vibrato[i] = 2.0 * ST_PI / period;
	}
	for( int i = 0; i < VEL_NUM; i++ )
		noteExtention[i] = pow( 2.0, ( 64.0 - i ) / 64.0 );
	vibrato[0] = 0.0;
}

VCNT_RESULT	vsqConverter::getData( string inputFileName, vConnectData& dstData )
{
	VCNT_RESULT			result;
	unsigned long		numFrames, beginFrame, noteFrameLength;
	vConnectFrame		frame;
	long				currentTick, beginTick, portamentoLength = 0, overshootLength = 0;
	double				currentSecond, f0Pos;
	double				vibratoDepth, vibratoOmega, vibratoTheta = 0.0, portamentoDepth = 1.0, overshootDepth = 1.0;
	vsqControl			control;
	string				voicePath;
	vsqEventProperty	notes;
	notes.present = notes.next = NULL;
	bool				isContinuousBack = false;

	frame.pronounces.resize( 2 );

	if( ( result = vsq.readVsqFile( inputFileName ) ) == VCNT_TRUE ){

#ifdef DEBUG
		cout << "Finished reading vsq meta text file: " << inputFileName << endl;
#endif

		vsq.getVoicePath( voicePath );

		/* Calculate beginFrame and numFrames here. */
		beginFrame = (long)( (double)( vsq.tickToSecond( ( beginTick = vsq.getBeginTick() ) ) ) * 1000.0 / config.frameShift );
		numFrames = (long)( (double)( vsq.tickToSecond( vsq.getEndTick() ) ) * 1000.0 / config.frameShift ) - beginFrame;

		dstData.setNumFrames( numFrames );

		frame.pronounces[0].stfFileName = "";
		frame.pronounces[1].stfFileName = "";

		for( unsigned long i = 0; i < numFrames; i++ ){
			currentSecond = (double)( beginFrame + i ) * config.frameShift / 1000.0;
			currentTick = vsq.secondToTick( currentSecond );

			if( notes.present == NULL || notes.presentEndTick < currentTick ){
				vsq.getEvents( currentTick, notes );
				overshootDepth = portamentoDepth; overshootLength = portamentoLength;
				vibratoTheta = 0.0;
				if( notes.present ){
					notes.presentBeginFrame = (unsigned long)( vsq.tickToSecond( notes.presentActualTick ) / config.frameShift * 1000.0 ) - beginFrame;
					notes.presentEndFrame = (unsigned long)( vsq.tickToSecond( notes.presentEndTick ) / config.f0FrameShift * 1000.0 ) - beginFrame;
					frame.pronounces[0].stfFileName = voicePath + notes.present->getStfFileName();
					if( frame.pronounces[0].stfFileName.compare( voicePath ) == 0 )
						frame.pronounces[0].stfFileName = "";
					frame.pronounces[0].morphRate = 0.0;
					noteFrameLength = notes.presentEndFrame - notes.presentBeginFrame;
					portamentoLength = notes.present->getPortamentoLength() * noteFrameLength;
					portamentoDepth = notes.present->getPortamentoDepth() * ( -3.0 );
				} else {
					portamentoLength = 0;
					portamentoDepth = 0.0;
				}

				if( notes.next ){
					notes.nextBeginFrame = (unsigned long)( vsq.tickToSecond( notes.nextActualTick ) / config.frameShift * 1000.0 ) - beginFrame;
					if( notes.nextBeginFrame < notes.presentBeginFrame )
						notes.nextBeginFrame = notes.presentBeginFrame;
					notes.nextEndFrame = (unsigned long)( vsq.tickToSecond( notes.nextEndTick ) / config.f0FrameShift * 1000.0 ) - beginFrame;
					if( notes.nextEndFrame < notes.presentEndFrame )
						notes.presentEndFrame = notes.nextEndFrame - 1;
					frame.pronounces[1].stfFileName = voicePath + notes.next->getStfFileName();
					frame.pronounces[1].morphRate = 0.0;
					isContinuousBack = ( notes.nextBeginFrame < notes.presentEndFrame + 50 );
					portamentoDepth = (double)( notes.nextNote - notes.presentNote ) * notes.presentPortamentoDepth;
				} else {
					frame.pronounces[1].stfFileName = "";
					isContinuousBack = false;
				}
			}

			vibratoOmega = 0.0;

			vsq.getControl( currentTick, control );
			setControl( frame, control );

			/* Calculate f0, dynamics and pronounces here. */
			if( notes.present && notes.presentActualTick <= currentTick && currentTick <= notes.presentEndTick ){
				frame.pronounces[0].morphRate = 1.0;
				calculatePronounce( frame.pronounces[0], *(notes.present), notes.presentBeginFrame, notes.presentEndFrame, notes.presentVelocity, notes.presentFixedLength, i );
				f0Pos = (double)( currentTick - notes.presentTick ) / (double)( notes.presentLength );

				vibratoOmega = vibrato[notes.present->getVibratoRate( currentTick )];
				vibratoDepth = notes.present->getVibratoDepth( currentTick );

				frame.f0 = noteFrequency[notes.presentNote];

				if( notes.next && 1.0 - f0Pos < notes.presentPortamentoLength && isContinuousBack ){
					double rate = ( 1.0 - f0Pos ) / notes.presentPortamentoLength;
					frame.f0 = pow( frame.f0, rate ) * pow( noteFrequency[notes.nextNote], 1.0 - rate );
					frame.f0 *= pow( 2.0, - portamentoDepth * sin( ST_PI * rate ) / 12.0 );
				}

				if( (long)i - (long)notes.presentBeginFrame < overshootLength / 2 )
					frame.f0 *= pow( 2.0, overshootDepth * sin( ST_PI * (double)( i- notes.presentBeginFrame ) / (double)overshootLength * 2 ) / 12.0 );

				if( notes.next && notes.nextActualTick < currentTick && currentTick <= notes.nextEndTick ){
					calculatePronounce( frame.pronounces[1], *(notes.next), notes.nextBeginFrame, notes.nextEndFrame, notes.nextVelocity, notes.nextFixedLength, i );
					frame.pronounces[1].morphRate = (double)( i - notes.nextBeginFrame ) / (double)( notes.presentEndFrame - notes.nextBeginFrame );
					frame.pronounces[0].morphRate = 1.0 - frame.pronounces[1].morphRate;
				}
				
				if( !isContinuousBack && (long)notes.presentEndFrame - (long)i < 50 ){
					frame.expression.volumeAmplitudeRate *= (double)( (long)notes.presentEndFrame - (long)i ) / 50.0;
					if( frame.expression.volumeAmplitudeRate < 0 )
						frame.expression.volumeAmplitudeRate = 0.0;
				}

			} else {
				frame.f0 = 0.0;
				frame.pronounces[0].morphRate = 0.0;
				frame.pronounces[1].morphRate = 0.0;
				vibratoTheta = 0.0;
				vibratoDepth = 0.0;
				vibratoOmega = 0.0;
				frame.expression.volumeAmplitudeRate = 0.0;
			}

			vibratoTheta += vibratoOmega * config.frameShift / 1000.0;

			frame.f0 *= pow( 2.0, 1.0 / 12.0 * vibratoDepth * sin( vibratoTheta ) );
			frame.f0 *= pow( 2.0, 1.0 / 12.0 * (double)control.pit / 8192.0 * control.pbs );
			frame.f0 *= getPitchFluctuation( currentSecond );

			dstData.setFrame( i, frame );
	
		}
		dstData.setOffset( (double)beginFrame * config.frameShift / 1000.0 );

#ifdef DEBUG
		cout << "Finished converting vsq meta text into v.Connect data." << endl;
#endif
	}

	return result;
}

void		vsqConverter::setControl( vConnectFrame& frame, vsqControl& control )
{
	frame.expression.volumeAmplitudeRate = (double)( control.dyn ) / 64.0;
	frame.expression.aperiodicityAmplitudeRate = 1.0 - (double)( control.bre ) / 144.0;
	frame.expression.spectrumExtentionRate = pow( 0.75, -1.0 + (double)( control.gen ) / 64.0 );
	frame.expression.dipFilters.resize( 2 );
	frame.expression.dipFilters[0].setFilter( 3000.0, 2.0, (double)( control.cle ) / 128.0 );
	frame.expression.dipFilters[1].setFilter( frame.f0, 2.0, 1.0 - (double)( control.bri ) / 64.0 );
	frame.expression.volumeAmplitudeRate /= frame.expression.aperiodicityAmplitudeRate;
}

double	vsqConverter::getPitchFluctuation( double second )
{
	double result = 1.0 + ( sin( 12.7 * ST_PI * second ) + sin ( 7.1 * ST_PI * second ) + sin( 4.7 * ST_PI * second ) / 3.0 ) / 300.0;

	return result;
}

void	vsqConverter::calculatePronounce( vConnectPronounce& pronounce, vsqEventEx& note, long noteBeginFrame, long noteEndFrame, char velocity, float fFixedLength, unsigned long frameIndex )
{
	long fixedLength = (unsigned long)( fFixedLength / config.frameShift );
	double scale = noteExtention[velocity]; //pow( 2.0, ( 64.0 - note.getVelocity() ) / 64.0 );

	pronounce.absolute = (unsigned long)( note.getLeftBlnak() / config.frameShift );
	if( (long)frameIndex - noteBeginFrame < fixedLength * scale ){
		pronounce.absolute += (long)( ( (long)frameIndex - (long)noteBeginFrame ) / scale );
		pronounce.relative = 0.0;
	} else {
		pronounce.absolute += fixedLength;
		pronounce.relative = ( (double)frameIndex - (double)noteBeginFrame - (double)fixedLength * scale )
							/ ( (double)noteEndFrame - (double)noteBeginFrame - (double)fixedLength * scale );
		if( pronounce.relative > 1.0 )
			pronounce.relative = 1.0;
	}
	pronounce.rightBlank = (unsigned long)( note.getRightBlank() / config.frameShift );
}
