/*======================================================*/
/*														*/
/*	vsqEventEx.cpp										*/
/*						(c) HAL 2009-					*/
/*														*/
/*======================================================*/
#include "vsqEventEx.h"

void vsqEventEx::setParameter( string left, string right )
{
    if( left.compare( "Type" ) == 0 )
        type = right;
    else if( left.compare( "Length" ) == 0 )
        length = atoi( right.c_str() );
    else if( left.compare( "Note#" ) == 0 )
        note = atoi( right.c_str() );
    else if( left.compare( "Dynamics" ) == 0 )
        velocity = atoi( right.c_str() );
    else if( left.compare( "PMBendDepth" ) == 0 )
        portamentoDepth = atoi( right.c_str() );
    else if( left.compare( "PMBendLength" ) == 0 )
        portamentoLength = atoi( right.c_str() );
    else if( left.compare( "PMbPortamentoUse" ) == 0 )
        portamentoUse = atoi( right.c_str() );
    else if( left.compare( "DEMdecGainRate" ) == 0 )
        decay = atoi( right.c_str() );
    else if( left.compare( "DEMaccent" ) == 0 )
        attack = atoi( right.c_str() );
    else if( left.compare( "VibratoDelay" ) == 0 )
        vibratoDelay = atoi( right.c_str() );
    else if( left.compare( "PreUtterance" ) == 0 )
        utauSetting.msPreUtterance = (float)atof( right.c_str() );
    else if( left.compare( "VoiceOverlap" ) == 0 )
        utauSetting.msVoiceOverlap = (float)atof( right.c_str() );
    else if( left.compare( "LyricHandle" ) == 0 ){
        right = "[" + right + "]";
        objectMap.insert( make_pair( right, (vsqBase*)&lyricHandle ) );
    }else if( left.compare( "VibratoHandle" ) == 0 ){
        right = "[" + right + "]";
        objectMap.insert( make_pair( right, (vsqBase*)&vibratoHandle ) );
	}else{
		string message = "warning: unknown Property in VsqEvent : " + left;
		outputError( message.c_str() );
	}

    return;
}

void vsqEventEx::applyPreUtterance( void )
{
	double positionSec = tempo.tickToSecond( tick ) - ( utauSetting.msPreUtterance / 1000.0 ) * pow( 2.0, (double)( 64 - velocity ) / 64.0 );
	actualTick = tempo.secondToTick( positionSec );
}

void vsqEventEx::setVoiceDataBase( vsqPhonemeDB& voiceDataBase )
{
	float preUtterance = utauSetting.msPreUtterance;
	float voiceOverlap = utauSetting.msVoiceOverlap;

	if( type.compare( "Anote" ) == 0 ){
		voiceDataBase.getUtauParameters( utauSetting, lyricHandle.getLyric() );
#ifdef	__UTAU_OFFSET__
		utauSetting.msPreUtterance += preUtterance;
		utauSetting.msVoiceOverlap += voiceOverlap;
#else
		utauSetting.msPreUtterance = preUtterance;
		utauSetting.msVoiceOverlap = voiceOverlap;
#endif
	}
}

double vsqEventEx::getVibratoDepth( long tick )
{
	double ret = 0.0;
	long vibBeginTick = this->tick + vibratoDelay;
	if( vibBeginTick <= tick ){
		double position = (double)( tick - vibBeginTick )/(double)( length - vibratoDelay );
		ret = (double)( vibratoHandle.getVibratoDepth( position ) ) * 2.5 / 127.0 / 2.0;
	}
	return ret;
}

int vsqEventEx::getVibratoRate( long tick )
{
	int ret = 0;
	long vibBeginTick = this->tick + vibratoDelay;
	if( vibBeginTick <= tick ){
		double position = (double)( tick - vibBeginTick )/(double)( length - vibratoDelay );
		ret = vibratoHandle.getVibratoRate( position );
	}
	return ret;
}
