/*======================================================*/
/*														*/
/*	vsqFileEx.cpp										*/
/*						(c) HAL 2009-					*/
/*														*/
/*======================================================*/
#include "vsqFileEx.h"

vsqFileEx::vsqFileEx()
{
	string temp;

	controlCurves.resize( CONTROL_CURVE_NUM );

	for( unsigned int i = 0; i < controlCurves.size(); i++ ){
		objectMap.insert( make_pair( controlCurveName[i], (vsqBase*)&(controlCurves[i]) ) );
		/* The note on 0 tick can be allocated before 0 tick, because of its preutterance. */
		controlCurves[i].setParameter( -10000, controlCurveDefaultValue[i] );
	}

	temp = "[EventList]";
	objectMap.insert( make_pair( temp, (vsqBase*)&events ) );
	temp = "[Tempo]";
	objectMap.insert( make_pair( temp, (vsqBase*)&tempo ) );
	temp = "[Common]";
	objectMap.insert( make_pair( temp, (vsqBase*)&common ) );
	temp = "[Master]";
	objectMap.insert( make_pair( temp, (vsqBase*)&master ) );
	temp = "[Mixer]";
	objectMap.insert( make_pair( temp, (vsqBase*)&mixer ) );
	temp = "[oto.ini]";
	objectMap.insert( make_pair( temp, (vsqBase*)&voiceDataBase ) );
	
}

VCNT_RESULT	vsqFileEx::readVsqFile( string fileName )
{
	VCNT_RESULT result = VCNT_FILE_NOT_FOUND;
	FILE* fp;
	char buf[LINEBUFF_LEN];

#ifdef __GNUC__
	fp = fopen( fileName.c_str(), "rb" );
#else
	fopen_s( &fp, fileName.c_str(), "rb" );
#endif

	if( fp ){
		string temp, search, left, right;
		MAP_TYPE<string,vsqBase*>::iterator i;

		while( c_getline( buf, LINEBUFF_LEN, fp ) != NULL ){
			temp = buf;
			if( temp.find( "[" ) == 0 ){
				search = temp;
				continue;
			}
			if( temp.find( "=" ) == string::npos ){
				left = temp;
				right = "";
			} else {
				left = temp.substr( 0, temp.find( "=" ) );
				right = temp.substr( temp.find( "=" ) + 1 );
			}
			i = objectMap.find( search );
			if( i != objectMap.end() ){
				if( i->second )
					i->second->setParameter( left, right );
			} else {
				cout << "error@vsqFileEx#readVsqFile; not found: " << search << endl;
			}
		}
		fclose( fp );
		if( events.empty() )
			result = VCNT_INVALID_CALL;
		else
			result = VCNT_TRUE;
		events.setVoiceDataBase( voiceDataBase );
		events.applyPreUtterance();
		beginTick = events.getBeginTick();
	}

	return result;
}

void		vsqFileEx::getControl( long tick, vsqControl& control )
{
	control.dyn = controlCurves[DYNAMICS].getValue( tick );
	control.bre = controlCurves[BRETHINESS].getValue( tick );
	control.bri = controlCurves[BRIGHTNESS].getValue( tick );
	control.cle = controlCurves[CLEARNESS].getValue( tick );
	control.gen = controlCurves[GENDER].getValue( tick );
	control.pit = controlCurves[PITCH_BEND].getValue( tick );
	control.pbs = controlCurves[PITCH_BEND_SENS].getValue( tick );
}

